"""Play With Docker Client - Create Command."""


from json import dumps
from .base import Base



import os
import sys

import requests

import yaml
from datetime import date, time, datetime, timedelta

from pprint import pprint
import logging
import re
from subprocess import Popen, PIPE
import click

from .. import utils


class Create(Base):
   

    def create_session(self):
        global session
        self.utils.blue("- create_session")
        resp = requests.post( self.utils.pwd_url )
        if resp.status_code != 200:
            print "Create Session error"
            raise BaseException('POST ' + self.utils.pwd_url + " = " + format(resp.status_code))
        else:
            try:
                id = re.search('.*/p/(.+)$', resp.url).group(1)
            except AttributeError:
                raise BaseException('session not found in ' + resp.url)
            print "Create Session ok " + self.utils._url('/p/'+id)
            self.utils.session['id'] = id
            pprint (self.utils.session)
        return self.utils.ok

    def create_instance(self, i):
        global session
        self.utils.blue("- create_instance " + str(i))
        resp = requests.post( self.utils._url("/sessions/{:s}/instances".format(self.utils.session['id'])) )

        if resp.status_code == 500:
            self.utils.rm_session()
            BaseException("Erreur 500, surement la session existe plus")
        elif resp.status_code == 409:
            print "Max number instances is reached"
            return self.utils.ko
        elif resp.status_code != 200:
            # This means something went wrong.
            resp.raise_for_status()
            raise BaseException('GET '+ self.utils.pwd_url + "/sessions/" + self.utils.session['id'] + "/instances = " \
                            + format(resp.status_code) + format(resp.headers))
        else:
            print('Created session ok with. IP: {}'.format(resp.json()["ip"]))        
            self.utils.session['ip'+str(i)] = resp.json()["ip"]
            res = self.utils.write_session()
            assert res == self.utils.ok
        return self.utils.ok


    def init_swarm(self):
        global session
        self.utils.blue ("- init_swarm")
        if not 'ip1' in self.utils.session:
            print "Session is not correctly initialised"
            return self.utils.ko

        os.environ['DOCKER_HOST'] = 'tcp://pwd' + self.utils.session["ip1"].replace(".", "_")  + '-2375.' + self.utils.pwd_dns + ':80'

        if self.options["-d"] == True:
            print "DOCKER_HOST="+os.environ['DOCKER_HOST']    
        try:
            out=""
            p = Popen('docker swarm init --advertise-addr %s' % (self.utils.session["ip1"]), shell=True, stdout=PIPE, stderr=PIPE)
            if p.wait() != 0:
                raise RuntimeError(p.stderr.read())            
            for line in p.stdout:
                if self.options["-d"] == True:
                    print ": "+line,
                if line.find("    ") != -1:
                    out += line
            if self.options["-d"] == True:
                print "out = '"+out.strip(' \t\r\n')+"'"
            self.utils.session['join'] = out.strip(' \t\r\n')
            self.utils.write_session()
        except RuntimeError as e:
            self.utils.red(str(e))

    def join_swarm(self, i):
        self.utils.blue ("- join_swarm - node" + str(i))

        if not 'ip'+str(i) in self.utils.session:
            print "Session is not correctly initialised"
            return self.utils.ko

        if not 'join' in self.utils.session:
            print "You must run init_swarm before adding a node"
            return self.utils.ko

        #Connect on node i
        os.environ['DOCKER_HOST'] = 'tcp://pwd' + self.utils.session["ip"+str(i)].replace(".", "_")  + '-2375.' + self.utils.pwd_dns + ':80'
        print "DOCKER_HOST="+os.environ['DOCKER_HOST']
        try:
            p = Popen(self.utils.session["join"], shell=True, stdout=PIPE, stderr=PIPE)
            if p.wait() != 0:
                raise RuntimeError(p.stderr.read())
                for line in p.stdout:
                    print "p: "+line,
        except RuntimeError as e:
            self.utils.red(str(e))

        
    def run(self):

        if self.options["-d"] == True:
            print('You supplied the following options:', dumps(self.options, indent=2, sort_keys=True))

        self.utils.ping()
    
        if not self.utils.read_session():
            if self.create_session():
                res = self.utils.write_session()
                assert res == self.utils.ok

        self.utils.debug("green", "session is " + self.utils.session['id'])
        if self.utils.options["-d"] == True:
            pprint (self.utils.session)

        if self.utils.check_session() == self.utils.ko:
            self.utils.red("Your session has been deleted, please retry now")
            return self.utils.ko

        for i in range(1,6):
            if not 'ip'+str(i) in self.utils.session:
                self.utils.blue("session 'ip"+str(i)+"' does not exists ==> create it")
                if not self.create_instance(i):
                    break                
            else:
                self.utils.debug("red", "session 'ip"+str(i)+"' already exists ==> do nothing")


        self.init_swarm()
        for i in range(2,6):
            self.join_swarm(i)

        self.utils.print_welcome()
                    




"""Tests for our `pwdc info` subcommand."""


from subprocess import PIPE, Popen as popen
from unittest import TestCase


class TestHello(TestCase):
    def test_returns_multiple_lines(self):
        output = popen(['pwdc', 'info'], stdout=PIPE).communicate()[0]
        lines = output.split('\n')
        self.assertTrue(len(lines) != 1)

    def test_returns_hello_world(self):
        output = popen(['pwdc', 'info'], stdout=PIPE).communicate()[0]
        self.assertTrue('No PWD Session, check --session_file parameter' in output)



"""Play With Docker Client - Status Command."""


from json import dumps
from .base import Base



import os
import sys

import requests

import yaml
from datetime import date, time, datetime, timedelta

from pprint import pprint
import logging
import re
from subprocess import Popen, PIPE
import click

from .. import utils


class Info(Base):
   
        

    def run(self):

        if self.options["-d"] == True:
            print('You supplied the following options:', dumps(self.options, indent=2, sort_keys=True))



        if self.utils.pwd_init() == self.utils.ko:
            return self.utils.ko

        self.utils.print_welcome()
        return self.utils.ok



"""Play With Docker Client - Init Command."""
""" used to setup the PWD url for your pwdc client """

from json import dumps
from .base import Base

import os
import sys

import requests

import yaml
from datetime import date, time, datetime, timedelta

from pprint import pprint
import logging
import re
from subprocess import Popen, PIPE
import click

from .. import utils


class Init(Base):
   
        

    def run(self):

        if self.options["-d"] == True:
            print('You supplied the following options:', dumps(self.options, indent=2, sort_keys=True))

        self.utils.write_config_file()
        self.utils.green("pwd_url is " + self.utils.pwd_url)




"""Play With Docker Client - attach Command."""


from json import dumps
from .base import Base



import os
import sys

import requests

import yaml
from datetime import date, time, datetime, timedelta

from pprint import pprint
import logging
import re
from subprocess import Popen, PIPE
import click

from .. import utils


def confirm(prompt=None, resp=False):
    """prompts for yes or no response from the user. Returns True for yes and
    False for no.

    'resp' should be set to the default value assumed by the caller when
    user simply types ENTER.

    >>> confirm(prompt='Create Directory?', resp=True)
    Create Directory? [y]|n: 
    True
    >>> confirm(prompt='Create Directory?', resp=False)
    Create Directory? [n]|y: 
    False
    >>> confirm(prompt='Create Directory?', resp=False)
    Create Directory? [n]|y: y
    True

    """
    
    if prompt is None:
        prompt = 'Confirm'

    if resp:
        prompt = '%s [%s]|%s: ' % (prompt, 'y', 'n')
    else:
        prompt = '%s [%s]|%s: ' % (prompt, 'n', 'y')
        
    while True:
        ans = raw_input(prompt)
        if not ans:
            return resp
        if ans not in ['y', 'Y', 'n', 'N']:
            print 'please enter y or n.'
            continue
        if ans == 'y' or ans == 'Y':
            return True
        if ans == 'n' or ans == 'N':
            return False   



class Create(Base):


    def attach_session(self):
        self.utils.blue("- attach_session")

        if self.options["--session_id"] != "":
            print "Attach Session " + self.options["--session_id"]
            self.utils.session['id'] = self.options["--session_id"]
            pprint (self.utils.session)
            res = self.utils.write_session()
            assert res == self.utils.ok
        return self.utils.ok



    def run(self):

        if self.options["-d"] == True:
            print('You supplied the following options:', dumps(self.options, indent=2, sort_keys=True))

        self.utils.ping()
    
        if (self.utils.read_session() == self.utils.ok) and (self.utils.check_session() == self.utils.ok):
            self.utils.debug("green", "you already have a session : " + self.utils.session['id'] + " in file : " + self.utils.session_file)
            if self.utils.options["-d"] == True:
                pprint (self.utils.session)
                if not confirm("Do you want to erase your session ?", False):
                    return self.utils.ko

        self.utils.rm_session()

        self.attach_session()

        self.utils.print_welcome()
                    




"""Play With Docker Client - Delete Command."""


from json import dumps
from .base import Base



import os
import sys

import requests

import yaml
from datetime import date, time, datetime, timedelta

from pprint import pprint
import logging
import re
from subprocess import Popen, PIPE
import click

from .. import utils


class Delete(Base):
   
        

    def delete_session(self):
        global session
        self.utils.blue("- delete_session")
        resp = requests.delete( self.utils._url("/sessions/{:s}".format(self.utils.session['id'])) )
        if resp.status_code != 204:
            if resp.status_code  == 404:
                print ("Session {:s} does not exist".format(self.utils.session['id']))
                return self.utils.ko
            else:
                print "Delete Session error"
                raise BaseException('Delete ' + self.utils.pwd_url + "/sessions/" + self.utils.session["id"])
        self.utils.rm_session()
        return self.utils.ok


    def run(self):

        if self.options["-d"] == True:
            print('You supplied the following options:', dumps(self.options, indent=2, sort_keys=True))

        if self.utils.pwd_init() == self.utils.ko:
            return self.utils.ko

        if not self.delete_session():
               self.utils.red("Error Deleting session")
               return self.utils.ko
        self.utils.blue("Session deleted, unset DOCKER_HOST and unset DOCKER_MACHINE_NAME")
        return self.utils.ok




"""
pwdc

Usage:
  pwdc init     [-d] --pwd-url=<url-of-pwd>
  pwdc create   [-d] [--session_file=<sf>] [--pwd-url=<url-of-pwd>]
  pwdc env      [-d] [-u] [--session_file=<sf>] [--pwd-url=<url-of-pwd>]
  pwdc delete   [-d] [--session_file=<sf>] [--pwd-url=<url-of-pwd>]
  pwdc info     [-d] [--session_file=<sf>] [--pwd-url=<url-of-pwd>]
  pwdc attach   [-d] --session_id=<si> [--pwd-url=<url-of-pwd>]
  pwdc -h | --help
  pwdc --version

Options:
  --session_file=<sf>               File were to store the pwd session [default: /tmp/pwd.session]
  --pwd-url=<url-of-pwd>            Play With Docker Domain Name
  -d                                Debug mode
  -u                                unset environment
  -h --help                         Show this screen.
  --version                         Show version.
  --session_id=<si>                 Attach to an existing session_id (this will replace your current session)

Examples:
  pwdc init --pwd-url=http://play-with-docker.com
  pwdc create                       Create a PWD session with Swarm enabled
  eval $(pwdc env)                  To Setup your docker client to call to your PWD instance
  pwdc delete                       Delete a PWD Session
  pwdc status                       Print the session status

Help:
  For help using this tool, please open an issue on the Github repository:
  https://github.com/allamand/pwd-cli
"""


from inspect import getmembers, isclass

from docopt import docopt

from . import __version__ as VERSION

from . import utils





#global utils

def main():
    """Main CLI entrypoint."""
    import pwdc.commands
    options = docopt(__doc__, version=VERSION)

    # Here we'll try to dynamically match the command the user is trying to run
    # with a pre-defined command class we've already created.
    for (k, v) in options.items(): 
        if hasattr(pwdc.commands, k) and v:
            module = getattr(pwdc.commands, k)
            pwdc.commands = getmembers(module, isclass)
            command = [command[1] for command in pwdc.commands if command[0] != 'Base'][0]

            u = utils.Utils(options) 
            command = command(u, options)

            command.run()




import sys
import os
from os.path import expanduser
import json
from pprint import pprint
import requests



class Utils(object):
    
    R="\033[31m"
    G="\033[32m"
    B="\033[36m"
    N="\033[m"

    ok=1
    ko=0

    config_file=expanduser("~")+"/.pwdc"
    config = {}

    #Default param will be overide by ~/.pwdc
    pwd_url="http://mustbeconfigured.com"

    session_file="./.session"
    session = {}

    def __init__(self, options):
        self.options = options
        self.session_file = options["--session_file"]

        self.check_config_file()

        self.debug("blue", "pwd_url is " + self.pwd_url)
        self.pwd_dns = self.pwd_url[self.pwd_url.find("://")+3:]


    def check_config_file(self):
        #Do we need to update config_file ?
        if self.options["--pwd-url"]:
             self.config["pwd_url"] = self.options["--pwd-url"]
             self.write_config_file()

        #Read Config file
        if not os.path.isfile(self.config_file):
            #init default
            self.red("You need configure pwdc with 'pwdc init --pwd-url=http://xxx' or configure '~/.pwdc'")
            #raise RuntimeError("configuration file ko ~/.pwdc")
            sys.exit(-1)
            #self.config["pwd_url"] = self.pwd_url
            #self.write_config_file()

        #Read file and init
        self.read_config_file()


    def read_config_file(self):
        with open(self.config_file) as f:
            self.config = json.load(f)
            if "pwd_url" in self.config:
                self.pwd_url = self.config["pwd_url"]
            else:
                if self.options["-d"] == True:
                    pprint(self.config)
                raise BaseException('no pwd_url in config ??? ') 


    def write_config_file(self):
        self.debug("blue", "Write_config_file" + self.options["--pwd-url"])
        with open(self.config_file, 'w') as f:
            json.dump(self.config, f)
        if self.options["-d"] == True:
            pprint(self.config)

## Print Messages

    def blue(self,msg):
        print self.B+msg+self.N
    def red(self,msg):
        print self.R+msg+self.N
    def green(self,msg):
        print self.G+msg+self.N

    print_color = {
        "blue"  : blue,
        "red"   : red, 
        "green" : green
    }
    def debug(self,c, msg):
        if self.options["-d"] == True :
            self.print_color[c](self, msg)



    def print_welcome(self):
        self.blue("1. You can View your session in a Browser")
        self.green(self.pwd_url+'/p/'+self.session['id'])
    
        if 'ip1' in self.session:
            print ""
            self.blue ("2. You can interract with your cluster bu executing :")
            self.green ('eval $(pwdc env)')
            self.blue ("Then just uses your classic docker client!!")


##Session Management


    def _url(self, path):
        return self.pwd_url + path

    def ping(self):
        self.blue("You are working with PWD : " + self.pwd_url)
        resp = requests.get( self._url("/ping") )
        assert resp.status_code == 200
#        self.debug("blue" ,"PWD server is UP :)")
        self.green(" --> PWD server is UP :)")


    def check_session(self):
        self.debug("blue", "- check_session")
        resp = requests.get( self._url("/sessions/" + self.session['id']) )
        if resp.status_code != 200:
            self.red ("Problem With session, you need to create NewOne")
            self.rm_session()
            return self.ko
        return self.ok



##Session File Management


    def read_session(self):
        self.debug("blue", "- read_session in file " + self.session_file)

        #path="."
        #for file in os.listdir(path):
            #if file.lower() == ".session" and os.path.isfile(os.path.join(path, file)):
            #with open(os.path.join(path, file)) as f:
        if os.path.isfile(self.session_file):
            with open(self.session_file) as f:
                self.session = json.load(f)
                if self.options["-d"] == True:
                    pprint(self.session)
                return self.ok
        return self.ko

    def write_session(self):
        self.debug("blue", "- write_session in file " + self.session_file)
        if self.options["-d"] == True:
            pprint(self.session)
        with open(self.session_file, 'w') as f:
            json.dump(self.session, f)
            return self.ok
        return self.ko

    def rm_session(self):
        self.blue("- rm_session file " + self.session_file)
        if os.path.isfile(self.session_file):
            os.remove(self.session_file)
        self.session= {}


    def pwd_init(self):

        self.ping()
    
        if not self.read_session():
            self.red("No PWD Session, check --session_file parameter")
            return self.ko

        if self.check_session() == self.ko:
            self.red("Your session has been deleted, please retry now")
            return self.ko

        self.green("session is " + self.session['id'])
        if self.options["-d"] == True:
            pprint (self.session)
        return self.ok



"""Tests for our `pwdc create` subcommand."""


from subprocess import PIPE, Popen as popen
from unittest import TestCase


#This test will only works if you have a valid PWD server in your ~/.pwdc
class TestCreate(TestCase):

    def test_create_ok(self):
        output = popen(['pwdc', 'create'], stdout=PIPE).communicate()[0]
        lines = output.split('\n')
        self.assertTrue(len(lines) != 1)
        self.assertTrue('You can interract with your cluster' in output)

        #info
        output = popen(['pwdc', 'info'], stdout=PIPE).communicate()[0]
        self.assertTrue('You can View your session in a Browser' in output)
        self.assertTrue('eval $(pwdc env)' in output)

        output = popen(['pwdc', 'env'], stdout=PIPE).communicate()[0]
        self.assertTrue('export DOCKER_HOST=' in output)
        self.assertTrue('export DOCKER_MACHINE_NAME=' in output)


        #Clean
        output = popen(['pwdc', 'delete'], stdout=PIPE).communicate()[0]
        self.assertTrue('Session deleted, unset DOCKER_HOST and unset DOCKER_MACHINE_NAME' in output)



    def with_no_session(self):


        output = popen(['pwdc', 'delete'], stdout=PIPE).communicate()[0]
        self.assertTrue('No PWD Session, check --session_file parameter' in output)


        output = popen(['pwdc', 'env'], stdout=PIPE).communicate()[0]
        self.assertTrue('No PWD Session, check --session_file parameter' in output)


        output = popen(['pwdc', 'info'], stdout=PIPE).communicate()[0]
        self.assertTrue('No PWD Session, check --session_file parameter' in output)







"""Tests for our main pwdc CLI module."""


from subprocess import PIPE, Popen as popen
from unittest import TestCase

from pwdc import __version__ as VERSION


class TestHelp(TestCase):
    def test_returns_usage_information(self):
        output = popen(['pwdc', '-h'], stdout=PIPE).communicate()[0]
        self.assertTrue('Usage:' in output)

        output = popen(['pwdc', '--help'], stdout=PIPE).communicate()[0]
        self.assertTrue('Usage:' in output)


class TestVersion(TestCase):
    def test_returns_version_information(self):
        output = popen(['pwdc', '--version'], stdout=PIPE).communicate()[0]
        self.assertEqual(output.strip(), VERSION)



"""Tests for our `pwdc create` subcommand."""


from subprocess import PIPE, Popen as popen
from unittest import TestCase


class TestCreate(TestCase):

    def test_returns_no_session(self):
        output = popen(['pwdc', 'env'], stdout=PIPE).communicate()[0]
        self.assertTrue('No PWD Session, check --session_file parameter' in output)



"""Packaging settings."""


from codecs import open
from os.path import abspath, dirname, join
from subprocess import call

from setuptools import Command, find_packages, setup

from pwdc import __version__


this_dir = abspath(dirname(__file__))
with open(join(this_dir, 'README.rst'), encoding='utf-8') as file:
    long_description = file.read()


class RunTests(Command):
    """Run all tests."""
    description = 'run tests'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        """Run all tests!"""
        errno = call(['py.test', '--cov=pwdc', '--cov-report=term-missing'])
        raise SystemExit(errno)


setup(
    name = 'pwdc',
    version = __version__,
    description = 'A simple python CLI for Play With Docker',
    long_description = long_description,
    url = 'https://github.com/allamand/pwdc',
    author = 'Allamand Sebastien',
    author_email = 'sebastien@allamand.com',
    license = 'UNLICENSE',
    classifiers = [
        'Intended Audience :: Developers',
        'Topic :: Utilities',
        'License :: Public Domain',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    keywords = 'cli',
    packages = find_packages(exclude=['docs', 'tests*']),
    install_requires = ['docopt', 'click'],
    extras_require = {
        'test': ['coverage', 'pytest', 'pytest-cov'],
    },
    entry_points = {
        'console_scripts': [
            'pwdc=pwdc.cli:main',
        ],
    },
    cmdclass = {'test': RunTests},
)



"""Play With Docker Client - Create Command."""


from json import dumps
from .base import Base


import os
import sys

parent_dir = os.path.abspath(os.path.dirname(__file__))
vendor_dir = os.path.join(parent_dir, 'vendor')
sys.path.append(vendor_dir)

import requests
import json
import yaml
from datetime import date, time, datetime, timedelta

from pprint import pprint
import logging
import re
from subprocess import Popen, PIPE
import click

from .. import utils



class Env(Base):


    def run(self):

        if self.options["-d"] == True:
            print('You supplied the following options:', dumps(self.options, indent=2, sort_keys=True))


        if self.options["-u"] == True:
            print ('unset DOCKER_HOST')
            print ('unset DOCKER_MACHINE_NAME')
            sys.exit(0)

#j'utilise pas le self.utils.pwd_init() car je veux pas de print (ca perturbe le eval)

        if not self.utils.read_session():
            self.utils.red("No PWD Session, check --session_file parameter")
            return self.utils.ko

        if self.utils.check_session() == self.utils.ko:
            self.utils.red("Your session has been deleted, please retry now")
            return self.utils.ko

#            raise RuntimeError("You must create a session")

        if 'ip1' in self.utils.session:
            print ('export DOCKER_HOST="tcp://pwd' + self.utils.session["ip1"].replace(".", "_") + '-2375.' + self.utils.pwd_dns + ':80"')
            print ('export DOCKER_MACHINE_NAME="'+self.utils.session['id']+'-'+self.utils.session["ip1"]+'"')
            return self.utils.ok
        else:
            pprint (self.utils.session)
            raise RuntimeError("ip1 does not exists in your PWD Session file.. " + self.utils.session_file)


